/*
 * Copyright (C) Horizon Robotics, Inc. - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 */

#ifndef DIAG_COMM_LIB_H_
#define DIAG_COMM_LIB_H_

#include <functional>
#include <iostream>
#include <map>
#include <string>
#include <vector>

namespace DiagnoseComm {

enum BoardType {
  soc_Pilot = 1,
  soc_ONE,
  soc_MAXFA,
  Default,
  FaultStatusMax = 100
};

enum ErrStatus { ERR_RECOVER = 2, ERR_HAPPEND = 3 };

struct DiagErrMsg {
  uint8_t soc_number;
  uint16_t module_id;
  uint16_t event_id;
  uint16_t dtc_id;
  uint8_t err_status;  // 2 for recover,3 for error
  uint8_t err_level;
  std::string dtc_description;
};

/**********************************
DiagFaultStatus
soc_number:0 for socA, 1 for socB, 2 for socC
err_status: 2 for recover , 3 for error
err_level: err level of fault
**********************************/
struct DiagFaultStatus {
  uint8_t soc_number;
  uint8_t err_status;
  uint8_t err_level;
};

struct DiagErrNode {
  // uint8_t soc_number;
  uint64_t timestamp_ms;
  uint16_t dtc_id;
  uint8_t err_status;
};
using DiagErrList = std::vector<DiagErrNode>;
using DiagErrListBuffer = std::vector<DiagErrList>;

struct DiagErrListStruct {
  uint8_t soc_number;
  DiagErrListBuffer err_list_buffer;
};

using SocNumber2DiagErrList =
    std::map<uint8_t, DiagnoseComm::DiagErrListBuffer>;

using DiagErrInfoCallback = std::function<void(const DiagErrMsg &)>;
using DiagFaultStatusCallback = std::function<void(const DiagFaultStatus &)>;
using DiagErrListCallback = std::function<void(const DiagErrListStruct &)>;
using DiagErrListStructCallback =
    std::function<void(const SocNumber2DiagErrList &)>;

int GetDiagnoseErrInfo(DiagErrInfoCallback cb);
int GetDiagnoseFaultStatus(DiagFaultStatusCallback cb);
/* need move to centered diag*/
int GetDiagnoseErrListPeriod(DiagErrListStructCallback cb, uint16_t interval_s);

}  // namespace DiagnoseComm

#endif