/*
 * Horizon Robotics
 *
 *  Copyright (C) 2019 Horizon Robotics Inc.
 *  All rights reserved.
 *  Author: leye.wang<leye.wang@horizon.ai>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
/* PRQA S ALL ++ */
#ifndef __HOBOT_DIAG_LOGGING_H__
#define __HOBOT_DIAG_LOGGING_H__

#include <stdio.h>
#include <stdlib.h>

#define STRINGIZE_NO_EXPANSION(x) #x
#define STRINGIZE(x) STRINGIZE_NO_EXPANSION(x)
#define HERE __FILE__ ":" STRINGIZE(__LINE__)


#define L_INFO "[INFO][" HERE "] "
#define L_WARNING "[WARNING]" HERE "] "
#define L_ERROR "[ERROR][" HERE "] "

#define thread_safe_log(level, fmt, ...)        \
  do {                                           \
      fprintf(stdout, level "" fmt, ##__VA_ARGS__); \
  } while(0);

#define NDEBUG
#ifndef NDEBUG
#define dlog(level, ...) log(level, __VA_ARGS__)
#define dlog_if(level, condition, ...) log_if(level, condition, __VA_ARGS__)
#else
#define dlog(...)
#define dlog_if(...)
#endif

#define log(level, ...) thread_safe_log(level, __VA_ARGS__)

#define log_if(level, condition, ...)                         \
    do {                                                      \
      if(condition) { thread_safe_log(level,  __VA_ARGS__); } \
    } while(0);

/* PRQA S ALL -- */
#endif // __HOBOT_LOGGING_H__
