#ifndef HB_ST_PERSISTENCY_H
#define HB_ST_PERSISTENCY_H


#include <sys/types.h>

#ifndef LOG_TAG
#define LOG_TAG "persist_log"

#endif

#define FILE_PATH_MAX_LEN 128

#define HB_PERSIST_OK							0
#define HB_PERSIST_ERROR_NORMAL					(-20000)
#define HB_PERSIST_ERROR_PARAM					(-20001)
#define HB_PERSIST_ERROR_NOACCESS				(-20002)

#define HB_PERSIST_ERROR_MALLOC					(-20003)
#define HB_PERSIST_ERROR_SOCKET					(-20004)
#define HB_PERSIST_ERROR_CONNECT				(-20005)
#define HB_PERSIST_ERROR_BIND					(-20006)
#define HB_PERSIST_ERROR_LISTEN					(-20007)
#define HB_PERSIST_ERROR_SEND					(-20008)
#define HB_PERSIST_ERROR_RECV					(-20009)

#define HB_PERSIST_ERROR_AUTH					(-20010)
#define HB_PERSIST_ERROR_UNSUPPORTALG			(-20011)

#define HB_PERSIST_ERROR_STORE					(-20012)
#define HB_PERSIST_ERROR_LOAD					(-20013)
#define HB_PERSIST_ERROR_DELETE					(-20014)

#define HB_PERSIST_ERROR_IMPORTKEY				(-20015)
#define HB_PERSIST_ERROR_CHECKPATH				(-20016)

typedef enum {
	HB_PERSIST_CMD_CONNECT = 0,
	HB_PERSIST_CMD_DCONNECT,
	HB_PERSIST_CMD_STORE,
	HB_PERSIST_CMD_LOAD,
	HB_PERSIST_CMD_DELETE
} persist_cmd_e;

typedef enum {
	PERSIST_CHIPER_ECB_AES_128,
	PERSIST_CHIPER_ECB_AES_192,
	PERSIST_CHIPER_ECB_AES_256,
	PERSIST_CHIPER_CBC_AES_128,
	PERSIST_CHIPER_CBC_AES_192,
	PERSIST_CHIPER_CBC_AES_256,
	PERSIST_CHIPER_CTR_AES_128,
	PERSIST_CHIPER_CTR_AES_192,
	PERSIST_CHIPER_CTR_AES_256
} hb_persist_chiper_type;

typedef enum {
	PERSIST_HASH_SHA1,
	PERSIST_HASH_SHA256,
	PERSIST_HASH_SHA512
} hb_persist_hash_type;
	
typedef struct {
	/* 客户端进程id,uid,gid */
	pid_t		pid;
	uid_t		uid;
	gid_t		gid;
	/* 客户端socket fd */
	int32_t 	client_fd;
	/* 分配给客户的id，用于内部唯一识别 */
	int32_t 	client_id;
	/* 服务端提供的服务对应msg命令字 */
	uint32_t	cmd;
	/* 返回值，小于0，表示失败，大于0 表示需要传输的数据长度 */
	int32_t 	ret;
	/* 客户端地址 */
	uint64_t	client_addr;
	/* 选择的加解密类型 */
	unsigned int chiper;
	/* 选择的hash类型 */
	unsigned int hash;
	/* 选择存储方式 */
	unsigned int secstore;
	/* 保留字段 */
	unsigned int reserved[4];
	/* 文件路径 */
	char path[FILE_PATH_MAX_LEN];
}st_msg_t;

typedef struct {

	/* 保留字段 */
	unsigned int reserved[8];

	/* 选择的加解密类型，参考 hb_persist_chiper_type */
	unsigned int chiper;

	/* 选择的hash类型 参考 hb_persist_hash_type */
	unsigned int hash;

	/* 输出的数据长度 */
	unsigned int outlen; 

	/* 输出的数据地址 */
	unsigned char *outbuf;
	/*
		如果路径为空，不走安全存储，普通加解密操作，加密接口hb_persist_encrypt_data会返回加密数据，解密接口hb_persist_decrypt_data会将返回解密数据。
		如果路径不为空，走安全存储，此时加解密接口相当于存储接口，hb_persist_encrypt_data进行加密存储，hb_persist_decrypt_data解密导出，并设置用户文件所属权限的访问限制。
	*/
	char path[FILE_PATH_MAX_LEN];
	/* 输入的加解密数据的长度 */
	unsigned int datalen;

	/* 输入的加解密数据存放地址 */
	unsigned char payload[0];
} hb_persist_meta_data;


/* 客户进程注册接口，会创建与服务进程的socket链接 */
int32_t hb_persist_register(void);

long hb_persist_encrypt_data(int32_t fd, hb_persist_meta_data *persistdata);

long hb_persist_decrypt_data(int32_t fd, hb_persist_meta_data *persistdata);

long hb_persist_delete_data(int32_t fd, const char *file_path);


/* 客户进程注销接口，会断开与服务进程的socket链接 */
long hb_persist_unregister(int32_t fd);

#endif  //HB_ST_PERSISTENCY_H

